//
// Lines starting with two slashes are comments, which are ignored by Stata
//
// A monte Carlo Simulation runs a program that calculates a statistic and this is
// repeated multiple times to see the behavior of the statistic over replications.
//
// The program below generates three multivariate normal variables and then generates a
// fourth variable which is a weighted sum of the three variables plus a normally 
// distributed error term. The data are then used to estimate a regression model and
// Stata collects the results from all replications as a dataset
//


// Drop earlier version of the program from memory if we have one
capture program drop OLS
// Drop also any prior graphs
capture graph drop _all

// The previous two commands need the capture prefix because they would result in errors
// if there was no OLS program or graphs in memory

program OLS
	
	// Generate data for independent variables by drawing from a correlation matrix
	
	matrix V = (1, .3, .3 \ .3, 1, .3 \ .3, .3, 1)
	drawnorm x1 x2 x3, n(200) corr(V) clear
	
	// Genrate the dependent variable using an equation
	
	generate y = 1*x1 + 2*x2 + 3*x3 + 5*rnormal()
	
	regress y x1 x2 x3
end

// Do a simulation with the program. Store estimates (_b) and standard errors (_se)

simulate _b _se, reps(1000): OLS


// Print summary statistics of the 1000 replications. This shows that the estimates are
// correct on average (unbiased) and that standard errors are unbiased estimates of the
// standard deviations of the estimates

summarize

// Show that estimates of X1 are normally distributed around the true value
// Unless we name a graph, it will be erased when the next graph is computed

kdensity _b_x1, normal xline(1) name("Estimates")

// Show that standard errors are unbiased estimates of the standard deviation of the
// estimates

quietly summarize _b_x1

// The text between ticks are Stata macro, which is substituted with a value before the
// command is run. In this case the value comes from the stored results (r) from the 
// summary command above

kdensity _se_x1, normal xline(`=r(sd)') name("StandardErrors")


